(ns rabbitmqquick.core)

(import (com.rabbitmq.client Consumer ConnectionFactory Connection Channel DefaultConsumer Envelope AMQP$BasicProperties AMQP$BasicProperties$Builder QueueingConsumer$Delivery))

(def connection-factory
     (doto (ConnectionFactory.) (.setHost "localhost")))

(def connection (.newConnection connection-factory))

(def channel (.createChannel connection))

(.queueDeclare channel "player_1" false false true nil)

(.queueDeclare channel "player_2" false false true nil)

(def ^{:const true}
  persistent-mode 2)

(defprotocol MessageMetadata
  (to-message-metadata [input] "Turns AMQP 0.9.1 message metadata into a Clojure map"))

(extend-protocol MessageMetadata
  com.rabbitmq.client.Envelope
  (to-message-metadata [input]
    {:delivery-tag (.getDeliveryTag input)
     :redelivery? (.isRedeliver input)
     :exchange (.getExchange input)
     :routing-key (.getRoutingKey input)})

  com.rabbitmq.client.AMQP$BasicProperties
  (to-message-metadata [input]
    {:content-type (.getContentType input)
     :content-encoding (.getContentEncoding input)
     :headers (.getHeaders input)
     :delivery-mode (.getDeliveryMode input)
     :persistent? (= persistent-mode (.getDeliveryMode input))
     :priority (.getPriority input)
     :correlation-id (.getCorrelationId input)
     :reply-to (.getReplyTo input)
     :expiration (.getExpiration input)
     :message-id (.getMessageId input)
     :timestamp (.getTimestamp input)
     :type (.getType input)
     :user-id (.getUserId input)
     :app-id (.getAppId input)
     :cluster-id (.getClusterId input)})
  
  com.rabbitmq.client.QueueingConsumer$Delivery
  (to-message-metadata [input]
    (merge (to-message-metadata (.getProperties input))
           (to-message-metadata (.getEnvelope input)))))

(defn ^Consumer create-default
  [^Channel channel &{:keys [handle-delivery-fn]}]
  (proxy [DefaultConsumer] [^Channel channel]
    (handleDelivery [^String consumer-tag ^Envelope envelope ^AMQP$BasicProperties properties ^bytes body]
      (when handle-delivery-fn
        (handle-delivery-fn channel (to-message-metadata (QueueingConsumer$Delivery. envelope properties body)) body)))))

(def guess-string-map {1 "Paper" 2 "Rock" 3 "Scissors"})

(defn getWinner [guess_1 guess_2]
  (if (= guess_1 guess_2)
    guess_1
    (if (= guess_2 (if (= guess_1 3) 1 (+ guess_1 1))) guess_1 guess_2)))

(def consumer (create-default channel
                              :handle-delivery-fn (fn [ch metadata ^bytes payload]
                                                    (if(get metadata :message-id)
                                                      (.basicPublish channel "" "player_1" (.build (doto (AMQP$BasicProperties$Builder.)
                                                                                                     (.messageId  (Integer/toString 1))
                                                                                                     (.timestamp (java.util.Date.))))
                                                                     (.getBytes (format "%s%d" (String. ^bytes payload) (+ (rand-int 3) 1))))))))

(def consumer_second (create-default channel
                                     :handle-delivery-fn (fn [ch metadata ^bytes payload]
                                                           (let [guesses (map str (String. ^bytes payload))]
                                                             (println "Player 1 Guessed" (get guess-string-map (Integer/parseInt (first guesses))))
                                                             (println "Player 2 Guessed" (get guess-string-map (Integer/parseInt (first (rest guesses)))))
                                                             (println "The winner is: " (guess-string-map (getWinner (Integer/parseInt (first guesses)) (Integer/parseInt (first (rest guesses))))))))
                                     :handle-consume-ok-fn (fn [consumer-tag])))

(.basicConsume channel "player_2" true consumer)
(.basicConsume channel "player_1" true consumer_second)

(dotimes [ i 10 ]
    (.basicPublish channel "" "player_2" (.build (doto (AMQP$BasicProperties$Builder.)
                                                   (.messageId  (Integer/toString i))
                                                   (.timestamp (java.util.Date.)))) (.getBytes (Integer/toString (+ (rand-int 3) 1)))))